unit IDEPlugin;

interface

uses
  Borland.Studio.ToolsAPI,
  Commons.Settings,
  Options,
  System.Collections,
  System.ComponentModel,
  System.Drawing,
  System.Resources,
  System.Windows.Forms;

type
  TIDEPlugin = class(System.ComponentModel.Component)
  {$REGION 'Designer Managed Code'}
  strict private
    /// <summary>
    /// Required designer variable.
    /// </summary>
    components: System.ComponentModel.IContainer;
    ImageList: System.Windows.Forms.ImageList;
    /// <summary>
    /// Required method for Designer support - do not modify
    /// the contents of this method with the code editor.
    /// </summary>
    procedure InitializeComponent;
  {$ENDREGION}
  strict protected
    procedure Dispose(Disposing: Boolean); override;
  strict private type
    TShutdownWizard = class(System.Object, IOTAWizard)
    strict private
      WizardIndex: Integer;
      Service: IOTAWizardService;
      FIDEShutdown: EventHandler;
    public
      function get_IDString: string;
      function get_Name: string;
    public
      property IDString: string read get_IDString;
      property Name: string read get_Name;
      property IDEShutdown: EventHandler add FIDEShutdown remove FIDEShutdown;
      procedure Execute;
      procedure Destroyed;
      constructor Create;
      destructor Destroy; override;
    end;
  strict private
    ShutdownWizard: TShutdownWizard;
    OTAService: IOTAService;
    MainMenuService: IOTAMainMenuService;
    MenuItemReformat,MenuItemInterface,MenuItemImplementation,
    MenuItemRemoveNonTOption,MenuItemTOption,MenuItemReload: IOTAMenuItem;
    Options: TOptions;
  strict private
    procedure IDEShutdownHandler(Sender: TObject; Args: EventArgs);
    procedure FileNotificationHandler(Sender: TObject; Args: FileNotificationEventArgs);
    procedure AdjustSelection(AddT,RemoveNonT,AddSystemT: Boolean);
    procedure OnMenuExecutedInterface(Sender: TObject; Args: EventArgs);
    procedure OnMenuExecutedImplementation(Sender: TObject; Args: EventArgs);
    procedure OnMenuExecutedTOption(Sender: TObject; Args: EventArgs);
    procedure OnMenuExecutedRemoveNonTOption(Sender: TObject; Args: EventArgs);
    procedure OnMenuExecutedReload(Sender: TObject; Args: EventArgs);
  public
    class procedure IDERegister; static;
    constructor Create;
  end;

  [assembly: RuntimeRequiredAttribute(TypeOf(TIDEPlugin))]

implementation

uses
  System.IO,
  System.Text;

{ TIDEPlugin.TShutdownWizard }

constructor TIDEPlugin.TShutdownWizard.Create;
begin
  inherited Create;
  Service := BorlandIDE.GetService(TypeOf(IOTAWizardService)) as IOTAWizardService;
  WizardIndex := Service.AddWizard(Self);
end;

destructor TIDEPlugin.TShutdownWizard.Destroy;
begin
  Service.RemoveWizard(WizardIndex);
  inherited Destroy;
end;

procedure TIDEPlugin.TShutdownWizard.Destroyed;
begin
  if Assigned(FIDEShutdown) then
    FIDEShutdown(Self,EventArgs.Create);
end;

procedure TIDEPlugin.TShutdownWizard.Execute;
begin
end;

function TIDEPlugin.TShutdownWizard.get_IDString: string;
begin
  Result := 'TIDEPlugin.PUses.TShutdownWizard';
end;

function TIDEPlugin.TShutdownWizard.get_Name: string;
begin
  Result := 'IDEPlugin PUses ShutdownWizard';
end;

{ TIDEPlugin }

{$REGION 'Windows Form Designer generated code'}
/// <summary>
/// Required method for Designer support - do not modify
/// the contents of this method with the code editor.
/// </summary>
procedure TIDEPlugin.InitializeComponent;
var
  resources: System.Resources.ResourceManager;
begin
  Self.components := System.ComponentModel.Container.Create;
  resources := System.Resources.ResourceManager.Create(TypeOf(TIDEPlugin));
  Self.ImageList := System.Windows.Forms.ImageList.Create(Self.components);
  // 
  // ImageList
  // 
  Self.ImageList.ImageSize := System.Drawing.Size.Create(16, 16);
  Self.ImageList.ImageStream := (System.Windows.Forms.ImageListStreamer(resources.GetObject('I' +
    'mageList.ImageStream')));
  Self.ImageList.TransparentColor := System.Drawing.Color.Transparent;
end;
{$ENDREGION}

class procedure TIDEPlugin.IDERegister;
begin
  TIDEPlugin.Create;
end;

procedure TIDEPlugin.Dispose(Disposing: Boolean);
begin
  if Disposing then begin
    if Components <> nil then
      Components.Dispose();
  end;
  inherited Dispose(Disposing);
end;

constructor TIDEPlugin.Create;
begin
  inherited Create;
  InitializeComponent;
  Options := TOptions.Load;
  MainMenuService := BorlandIDE.GetService(typeof(IOTAMainMenuService)) as IOTAMainMenuService;
  MainMenuService.AddMenuItem('HTMLTidySubmenuItem',OTAMenuItemLocation.otamlAfter,'UsesMenu','Uses Reformat');
  MenuItemReload := MainMenuService.AddMenuItem('UsesMenu',OTAMenuItemLocation.otamlChild,'UsesReloadItem','Reload &Options');
  MenuItemRemoveNonTOption := MainMenuService.AddMenuItem('UsesMenu',OTAMenuItemLocation.otamlChild,'UsesRemoveNonTOptionItem','&Non Prefix Units Removal',BitMap(ImageList.Images.Item[2]).GetHBitMap);
  MenuItemTOption := MainMenuService.AddMenuItem('UsesMenu',OTAMenuItemLocation.otamlChild,'UsesTOptionItem','&Prefix Units Insertion');
  MenuItemImplementation := MainMenuService.AddMenuItem('UsesMenu',OTAMenuItemLocation.otamlChild,'UsesImplementationItem','Reformat Im&plementation Uses',BitMap(ImageList.Images.Item[1]).GetHBitMap);
  MenuItemInterface := MainMenuService.AddMenuItem('UsesMenu',OTAMenuItemLocation.otamlChild,'UsesInterfaceItem','Reformat &Interface Uses',BitMap(ImageList.Images.Item[0]).GetHBitMap);
  MenuItemReformat := MainMenuService.AddMenuItem('UsesMenu',OTAMenuItemLocation.otamlChild,'UsesReformatItem','&Reformat Uses',BitMap(ImageList.Images.Item[0]).GetHBitMap);
  MenuItemRemoveNonTOption.Checked := Options.RemoveUnprefixedDefault;
  MenuItemTOption.Enabled := (Options.KnownNamespaceHash.Count > 0);
  MenuItemTOption.Checked := Options.AddPrefixDefault and MenuItemTOption.Enabled;
  MenuItemRemoveNonTOption.Visible := MenuItemTOption.Checked;
  MenuItemReformat.Visible := not MenuItemTOption.Checked;
  MenuItemInterface.Visible := not MenuItemReformat.Visible;
  MenuItemImplementation.Visible := not MenuItemReformat.Visible;
  Include(MenuItemReload.Executed, OnMenuExecutedReload);
  Include(MenuItemTOption.Executed, OnMenuExecutedTOption);
  Include(MenuItemRemoveNonTOption.Executed, OnMenuExecutedRemoveNonTOption);
  Include(MenuItemInterface.Executed, OnMenuExecutedInterface);
  Include(MenuItemImplementation.Executed, OnMenuExecutedImplementation);
  Include(MenuItemReformat.Executed, OnMenuExecutedImplementation);
  ShutdownWizard := TIDEPlugin.TShutdownWizard.Create;
  OTAService := BorlandIDE.GetService(typeof(IOTAService)) as IOTAService;
  Include(OTAService.FileNotification,FileNotificationHandler);
  Include(ShutdownWizard.IDEShutdown,IDEShutdownHandler);
end;

procedure TIDEPlugin.IDEShutdownHandler(Sender: TObject; Args: EventArgs);
begin
  Exclude(OTAService.FileNotification,FileNotificationHandler);
  MainMenuService.RemoveMenuItem('UsesReformatItem');
  MainMenuService.RemoveMenuItem('UsesInterfaceItem');
  MainMenuService.RemoveMenuItem('UsesImplementationItem');
  MainMenuService.RemoveMenuItem('UsesTOptionItem');
  MainMenuService.RemoveMenuItem('UsesRemoveNonTOptionItem');
  MainMenuService.RemoveMenuItem('UsesReloadItem');
  MainMenuService.RemoveMenuItem('UsesMenu');
  ShutdownWizard.Free;
end;

procedure TIDEPlugin.FileNotificationHandler(Sender: TObject; Args: FileNotificationEventArgs);

  function ConverShortCut(ShortCut: Keys): Integer;
  const
    KeyShift = $2000;
    KeyControl = $4000;
    KeyAlt = $8000;
  begin
    Result := 0;
    if (Keys.Control and ShortCut) <> Keys.None then
      Result := Result or KeyControl;
    if (Keys.Shift and ShortCut) <> Keys.None then
      Result := Result or KeyShift;
    if (Keys.Alt and ShortCut) <> Keys.None then
      Result := Result or KeyAlt;
    ShortCut := ShortCut and Keys.KeyCode;
    Result := Result or Convert.ToInt32(ShortCut);
  end;

begin
  if ((Args.NotifyCode = OTAFileNotification.ofnPackageInstalled) or
			(Args.NotifyCode = OTAFileNotification.ofnPackageUninstalled)  or
			(Args.NotifyCode = OTAFileNotification.ofnFileOpened)) then begin
    if Options.ItemImplementationShortcut <> Keys.None then
      MenuItemImplementation.ShortCut := ConverShortCut(Options.ItemImplementationShortcut);
    if Options.ItemInterfaceShortcut <> Keys.None then
      MenuItemInterface.ShortCut := ConverShortCut(Options.ItemInterfaceShortcut);
    if Options.ItemImplementationShortcut <> Keys.None then
      MenuItemReformat.ShortCut := ConverShortCut(Options.ItemImplementationShortcut);
    if Options.ItemRemoveNonPrefixShortCut <> Keys.None then
      MenuItemRemoveNonTOption.ShortCut := ConverShortCut(Options.ItemRemoveNonPrefixShortCut);
  end;
end;

procedure TIDEPlugin.AdjustSelection(AddT,RemoveNonT,AddSystemT: Boolean);
var
  ModuleServices: IOTAModuleServices;
  Module: IOTAModule;
  Editor: IOTAEditor;
  SourceEditor: IOTASourceEditor;
  View: IOTAEditView;
  Reader: IOTAFileReader;
  Writer: IOTAFileWriter;
  BlockStart: OTACharPos;
  BlockAfter: OTACharPos;
  StartPos, EndPos: LongInt;
  I: Integer;
  S: string;

  procedure AdjustSelectedText(var SelectedText: string);
  var
    UsesItems: array of string;
    UsesItemList,UpperUsesItemList: &ArrayList;
    SB: &StringBuilder;
    I: Integer;
    S: string;
    UsesSelected: Boolean;
  begin
    UsesItemList := ArrayList.Create;
    UpperUsesItemList := ArrayList.Create;
    SB := &StringBuilder.Create;
    UsesSelected := false;
    UsesItems := SelectedText.Split([' ',',',';']);
    for I := 0 to Pred(Length(UsesItems)) do begin
      S := UsesItems[I].Trim;
      if S <> '' then begin
        if S.ToUpper = 'USES' then
          UsesSelected := true
        else begin
          UsesItemList.Add(S);
          UpperUsesItemList.Add(S.ToUpper);
        end;
      end;
    end;
    if AddT then begin
      for I := 0 to Pred(UpperUsesItemList.Count) do begin
        S := string(UpperUsesItemList[I]);
        if not S.EndsWith(Options.UnitPostfix) then begin
          if Options.KnownNamespaceHash.ContainsKey(S) then begin
            if not UpperUsesItemList.Contains(S + Options.UnitPostfix) then
              UsesItemList.Add(string(UsesItemList[I]) + Options.UnitPostfix);
            if RemoveNonT then
              UsesItemList[I] := '';
          end;
        end
        else begin
          S := S.Substring(0,S.Length - Options.UnitPostfix.Length);
          if Options.KnownNamespaceHash.ContainsKey(S) then begin
            if not UpperUsesItemList.Contains(S) and not RemoveNonT then
              UsesItemList.Add(string(UsesItemList[I]).Substring(0,S.Length));
          end;
        end;
      end;
      if AddSystemT then begin
        if not UpperUsesItemList.Contains('SYSTEM' + Options.UnitPostfix) then
          UsesItemList.Add('System' + Options.UnitPostfix);
      end
      else begin
        I := UpperUsesItemList.IndexOf('SYSTEM' + Options.UnitPostfix);
        if I >= 0 then
          UsesItemList.RemoveAt(I);
      end;
    end
    else begin
      for I := Pred(UpperUsesItemList.Count) downto 0 do begin
        S := string(UpperUsesItemList[I]);
        if S.EndsWith(Options.UnitPostfix) then begin
          S := S.Substring(0,S.Length - Options.UnitPostfix.Length);
          if UpperUsesItemList.Contains(S) or (S = 'SYSTEM') then begin
            UpperUsesItemList.RemoveAt(I);
            UsesItemList.RemoveAt(I);
          end
          else begin
            UpperUsesItemList[I] := S;
            S := string(UsesItemList[I]);
            UsesItemList[I] := S.Substring(0,S.Length - Options.UnitPostfix.Length);
          end;
        end;
      end;
    end;
    UsesItemList.Sort;
    if UsesSelected then begin
      SB.Append('uses');
      SB.Append(&Environment.NewLine);
    end;
    for I := 0 to Pred(UsesItemList.Count) do begin
      if string(UsesItemList[I]) <> '' then begin
        SB.Append(Options.UsesIndent);
        SB.Append(UsesItemList[I]);
        SB.Append(',');
        SB.Append(&Environment.NewLine);
      end;
    end;
    SB.Chars[SB.Length - 1 - Environment.NewLine.Length] := ';';
    SelectedText := SB.ToString;
  end;

begin 
  ModuleServices := BorlandIDE.GetService(typeof(IOTAModuleServices)) as IOTAModuleServices;
  Module := ModuleServices.CurrentModule;
  if not Assigned(Module) then
    Exit;
  for I := 0 to Pred(Module.ModuleFileCount) do begin
    Editor := Module.ModuleFileEditors(I);
    if Editor is IOTASourceEditor then begin
      SourceEditor := IOTASourceEditor(Editor);
      Break;
    end;
  end;
  if not Assigned(SourceEditor) then
    Exit;
  if SourceEditor.EditViewCount > 1 then
    raise Exception.Create('Too many views open, please close all but one');
  if SourceEditor.EditViewCount < 1 then
    Exit;
  View := SourceEditor.GetEditView(0);
  if SourceEditor.BlockType = OTABlockType.btColumn then
    Exit;
  try
    BlockStart := SourceEditor.BlockStart;
    BlockAfter := SourceEditor.BlockAfter;
    BlockStart.CharIndex := 0;
    if BlockAfter.CharIndex > 0 then begin
      BlockAfter.CharIndex := 0;
      Inc(BlockAfter.Line);
    end;
    StartPos := View.CharPosToPos(BlockStart);
    EndPos := View.CharPosToPos(BlockAfter);
    if (EndPos - StartPos - 1) > 0 then begin
      Reader := SourceEditor.CreateReader;
      Reader.Read(StartPos,0);
      S := System.Text.Encoding.UTF8.GetString(Reader.Read(EndPos - StartPos - 1,0));
      Reader.Close;
      AdjustSelectedText(S);
      Writer := SourceEditor.CreateWriter;
      Writer.CopyTo(StartPos);
      Writer.DeleteTo(EndPos);
      Writer.Insert(S);
      Writer.Close;
      (SourceEditor as IOTAEditor).MarkModified;
    end;
  except
    ;
  end;
  (SourceEditor as IOTAEditor).Show;
end;

procedure TIDEPlugin.OnMenuExecutedInterface(Sender: TObject; Args: EventArgs);
begin
  AdjustSelection(MenuItemTOption.Checked,MenuItemRemoveNonTOption.Checked,true);
end;

procedure TIDEPlugin.OnMenuExecutedImplementation(Sender: TObject; Args: EventArgs);
begin
  AdjustSelection(MenuItemTOption.Checked,MenuItemRemoveNonTOption.Checked,false);
end;

procedure TIDEPlugin.OnMenuExecutedTOption(Sender: TObject; Args: EventArgs);
begin
  MenuItemTOption.Checked := not MenuItemTOption.Checked;
  MenuItemRemoveNonTOption.Visible := MenuItemTOption.Checked;
  MenuItemReformat.Visible := not MenuItemTOption.Checked;
  MenuItemInterface.Visible := not MenuItemReformat.Visible;
  MenuItemImplementation.Visible := not MenuItemReformat.Visible;
end;

procedure TIDEPlugin.OnMenuExecutedRemoveNonTOption(Sender: TObject; Args: EventArgs);
begin
  MenuItemRemoveNonTOption.Checked := not MenuItemRemoveNonTOption.Checked;
end;

procedure TIDEPlugin.OnMenuExecutedReload(Sender: TObject; Args: EventArgs);
begin
  Options := TOptions.Load;
  MenuItemTOption.Enabled := (Options.KnownNamespaceHash.Count > 0);
  MenuItemTOption.Checked := MenuItemTOption.Checked and MenuItemTOption.Enabled;
  MenuItemRemoveNonTOption.Visible := MenuItemTOption.Checked;
  MenuItemReformat.Visible := not MenuItemTOption.Checked;
  MenuItemInterface.Visible := not MenuItemReformat.Visible;
  MenuItemImplementation.Visible := not MenuItemReformat.Visible;
end;


end.
